#!/bin/bash
#
# rsync-now:bin/apis/rsync-now-common-config-api.sh
#
declare -gr _RSYNC_NOW_CONFIG_HOME_="${HOME}/.config/rsync-now"

function get_configuration_json_filename { # get_configuration_json_file
  # shellcheck disable=SC2154 # Must be define in main function
  if [ -n "${param_rsync_custom_json_config_name}" ] ; then
    local -r file="${_RSYNC_NOW_CONFIG_HOME_}/${param_rsync_custom_json_config_name}.json"
  else
    local -r file="${_RSYNC_NOW_CONFIG_HOME_}/rsync-now.json"
  fi

  echodebug "${FUNCNAME[0]}() => '${file}'"
  echo "${file}"
}

function _get_configuration_json_filename_if_exists_ {
  local json_file=
  json_file="$( get_configuration_json_filename )" || return $?

  if [ -f "${json_file}" ] ; then
    echoinfo_quiet "Configuration file: ${json_file}" >&2
    echo "${json_file}"
  else
    echofatal "Configuration file not found: '${json_file}'"
    return 100
  fi
}

function get_configuration_json_if_exists {
  local json_file=
  json_file="$( _get_configuration_json_filename_if_exists_ )" || return $?

  cat "${json_file}" || return $?
}

function get_value_from_json {
  local -r json_path="$1"
  local -r json="$2"

  local raw_varvalue=
  raw_varvalue="$( jq -r "${json_path}" <<<"${json}" )" || return $?
  if [ "${raw_varvalue}" = 'null' ] ; then
    return 100
  else
    echo "${raw_varvalue}"
  fi
}

function get_substituted_value_from_json {
  local -r json_path="$1"
  local -r json="$2"

  local raw_varvalue=
  raw_varvalue="$( get_value_from_json "${json_path}" "${json}" )" || return $?

  envsubst <<<"${raw_varvalue}"
  #echo "'${json_path}' -> '${raw_varvalue}'" >&2
}

function get_rsync_configs {
  local -r _json_config_="$1"

  jq -c '.sync // [] | .[]' <<<"${_json_config_}" || return $?
}

function get_vars_config {
  local -r _json_config_="$1"

  jq -c '.declare // empty' <<<"${_json_config_}" || return $?
}

function get_options_json_config {
  local -r _sync_entry_json_="$1"

  local _opts_=
  _opts_="$( get_value_from_json '.options // { exclude: [] , raw: [] }' "${_sync_entry_json_}" )" || return $?

  jq -c '.' <<<"${_opts_}" || return $?
}

function get_substituted_options_json_config {
  local -r _sync_entry_json_="$1"

  local _opts_=
# _opts_="$( get_substituted_value_from_json '.options // []' "${_sync_entry_json_}" )" || return $?
  _opts_="$( get_substituted_value_from_json '.options // { exclude: [] , raw: [] }' "${_sync_entry_json_}" )" || return $?

  jq -c '.' <<<"${_opts_}" || return $?
}

function get_options_excludes {
  local -r _options_json_="$1"

  local exclude_type=
  exclude_type="$( jq -r '.exclude | type' <<<"${_options_json_}" )" || return $?
  local -r exclude_type
  echodebug "exclude_type='${exclude_type}'"

  if [ "${exclude_type}" != 'array' ] ; then
    echowarn '.exclude not an array (skipping)'
    return 0
  fi

  local result=
  result="$( jq -r '.exclude[]' <<<"${_options_json_}" )" || return $?

  grep -v '^$' <<<"${result}" || true
}

function get_options_for_rsync {
  local -r _options_object_json_="$1"

  local json_options_type=
  json_options_type="$( jq -r 'type' <<<"${_options_object_json_}" )" || return $?
  local -r json_options_type

  local _opt_=
  case "${json_options_type}" in
    'array')
      # Configuration format v1: DEPRECATED
      local result=
      result="$( jq -r '.[]' <<<"${_options_object_json_}" )" || return $?

      grep -v '^$' <<<"${result}" || true
      ;;

    'object')
      # Configuration format v2
      local exclude_entries=
      exclude_entries="$( get_options_excludes "${_options_object_json_}" )" || return $?
      local -r exclude_entries

      while read -r _opt_ ; do
        if [ -n "${_opt_}" ] ; then
          echo --exclude
          echo "${_opt_}"
        fi
      done <<<"${exclude_entries}"

      local raw_type=
      raw_type="$( jq -r '.raw | type' <<<"${_options_object_json_}" )" || return $?
      local -r raw_type
      echodebug "raw_type='${raw_type}'"

      if [ "${raw_type}" != 'array' ] ; then
        echowarn '.raw not an array (ignore)'
      else
        while read -r _opt_ ; do
          if [ -n "${_opt_}" ] ; then
            echo "${_opt_}"
          fi
        done < <( jq -r '.raw[]' <<<"${_options_object_json_}" )
      fi
      ;;

    *)
      echowarn "Ignore configuration (type:'${_options_object_json_}')"
      ;;
  esac
}

function rsync_now_load_vars {
  local -r _json_config_="$1"

  export HOSTNAME # required by envsubst (but not for USER, why ?)
  export USER     # export any way, some environement could need it

  local _vars_json_=
  _vars_json_="$( get_vars_config "${_json_config_}" )" || return $?
  local -r _vars_json_

  load_vars_from_json_string "${_vars_json_}" || return $?
}
